<?php
namespace App\Http\Controllers\adminpanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
//use Mail;
use Illuminate\Support\Facades\Mail;
use App\Models\User;
use App\Models\Country;
use Illuminate\Support\Facades\Validator;
use Illuminate\Pagination\Paginator;
use Carbon\Carbon;
use Illuminate\Validation\Rule;

class UserController extends Controller
{
    private $pagination = 10;

    public function manage() {
        $data = User::paginate($this->pagination);
        return view('adminpanel.manageuser', compact('data'));
    }

    public function search(Request $request) {
        $input = $request->all();
        $qry = User::query();
        if(trim($input["search"])!="") {
            $search = $input["search"];
            $qry->where([
                ["username", "like", "%{$search}%"],
            ]);
            $qry->orwhere([
                ["email", "like", "%{$search}%"],
            ]);
        }
        $data = $qry->paginate($this->pagination);
        $data->appends($input);
        return view('adminpanel.manageuser', compact('data'));
    }

    public function add()
    {
        $data = array('type'=>'add');
        $country = Country::get()->pluck('country_name', 'id')->toArray();
        return view('adminpanel.adduser', compact('data', 'country'));
    }

    public function save(Request $request) {
        $input = $request->all();

        $country = Country::get()->pluck('country_name', 'id')->toArray();

        $validator = Validator::make( $input, $this->getRules('Add', $input), $this->messages());
        if ($validator->fails()) {
            $data = array('type'=>'add', 'input'=>$input, 'error'=>$validator->errors());
            return view('adminpanel.adduser', compact('data', 'country'));
            exit();
        }
        $input['password'] = bcrypt($input['password']);

        $user = User::create($input);
        if($user->id>0)
        {
            $mail = $input['email'];
            $data['username']=$user->username;

            Mail::send('adminpanel.welcomeEmail', $data,function($message) use($mail)
            {
                $message->from('kitest@keshavinfotechdemo2.com', 'Abhi-Game');
                $message->to($mail)->subject('Welcome to Abhi-Game!');
            });

            return redirect()->route('adminpanel.user.manage')->with('success', 'User Created successfully.');
        } else {
            return redirect()->route('adminpanel.user.add')->withErrors(['Error creating record. Please try again.']);
        }
    }

	public function edit($id)
    {
        $input = User::where('id', '=', $id)->get();
        $data = array('type'=>'edit', 'input'=>$input);

        $country = Country::get()->pluck('country_name', 'id')->toArray();
	    return view('adminpanel.adduser', compact('data', 'country'));
	}

	public function update(Request $request)
    {
		$input = $request->all();
        $id = $input['id'];

        $country = Country::get()->pluck('country_name', 'id')->toArray();
        $validator = Validator::make( $input, $this->getRules('Edit', $input), $this->messages());

        if ($validator->fails()) {
            $data = array('type'=>'Edit', 'input'=>$input, 'error'=>$validator->errors());
            return view('adminpanel.adduser', compact('data', 'country'));
            exit();
        }
        $update = array();
        $update["username"] = $input['username'];
        $update["countryId"] = $input['countryId'];
        $update["contactNumber"] = $input['contactNumber'];
        $update["email"] = $input['email'];

        if(isset($input['password']))
        {
            $update["password"] = bcrypt($input['password']);
        }

        $user = User::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.user.manage')->with('success', 'User Updated successfully.');
	}

    public function delete($id)
    {
        $user = User::where('id', $id)->delete();

        return redirect()->route('adminpanel.user.manage')->with('success', 'User Deleted successfully.');
    }

    private function getRules($type, $input)
    {
        $return = array();
        $return['username'] = [
            'required',
            'max:30',
            Rule::unique('users', 'username')->where(function ($query) use ($input) {
                $query->where('id', '!=', $input['id']);
            })
        ];

        $return['contactNumber'] = [
            'required',
            'max:20',
            function ($attribute, $value, $fail) use ($input)
            {
                if (User::where('contactNumber', $value)
                    ->where('id', '!=', $input['id'])
                    ->exists()) {
                    $fail('The contact number is already in used.');
                }
            }
        ];
        $return['email'] = [
            'required',
            'email',
            'max:100',
            Rule::unique('users', 'email')->where(function ($query) use ($input) {
                $query->where('id', '!=', $input['id']);
            })
        ];

        return $return;
    }

    private function messages() {
        return [
            'username.required'  => $this->getRequiredMessage('username'),
            'username.max'  => $this->getGreaterMessage('username', 30),
            'username.unique' => 'The username has already been taken.',
            'contactNumber.required'  => $this->getRequiredMessage('contact number'),
            'contactNumber.max'  => $this->getGreaterMessage('contact number', 20),
            'email.required'  => $this->getRequiredMessage('email'),
            'email.max'  => $this->getGreaterMessage('email', 100),
            'email.unique' => 'The email address has already been taken.',
        ];
    }

    private function getRequiredMessage($string) {
        return 'The ' . $string . ' field is required.';
    }

    private function getGreaterMessage($string, $maxchar) {
        return 'The ' . $string . ' may not be greater than ' . $maxchar . ' characters.';
    }
}

?>
