<?php

namespace App\Http\Controllers\api;

use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\User_purchase;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use Carbon\Carbon;
use Tymon\JWTAuth\Facades\JWTAuth;

class AuthController extends Controller
{
    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'username'          => 'required|min:3',
            'email'             => 'required',
            'password'          => 'required|min:6',
        ]);

        if ($validator->fails())
        {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();

        $input['password'] = bcrypt($input['password']);
        $input['user_token'] = md5(rand(1, 999999));

        $email = $input['email'];

        if(isset($input['email']))
        {
            if ($this->isUserExists($email))
            {
                return response()->json(['status' => 0, 'message' => 'User already registered with this email']);
            }
        }

        $user = User::create($input);

        if ($user)
        {
            $userData = Controller::getUserData($user);
            return response()->json(['status' => 1, 'message' => 'User Registered Successfully', 'result' => $userData]);
        }
        else
        {
            return response()->json(['status' => 0, 'message' => 'User not registered']);
        }
    }

    private function isUserExists($email)
    {
        return User::where('email', $email)->exists();
    }

    public function login(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email'     =>'required|email',
            'password'  =>'required',
        ]);

        $email = $request->input('email');
        $password = $request->input('password');

        if ($validator->fails())
        {
            return response()->json(['error'=>$validator->errors()], 403);
        }

        if (Auth::guard('web')->attempt(['email' => $email, 'password' => $password], false))
        {
            $user = Auth::user();
            $userData = Controller::getUserData($user);
            
            $JWTtoken = JWTAuth::fromUser($user);

            $tkn = md5(rand(1, 999999));
            $updateData['user_token'] = $tkn;
            User::where('user_id',$user->user_id)->update($updateData);

            return response()->json([
                'status' => 1, 
                'message' => "Login Successfully",
                'result' => $userData,
                'JWTtoken' => $JWTtoken
            ]);
        }
        return response()->json(['status' => 0, 'message' => "Username or Password is incorrect. Please try again!"]);
    }

    public function autoLogin(Request $request)
    {
        try {
            if (!$token = JWTAuth::parseToken()) {
                return response()->json(['status' => 0, 'message' => 'Token not found'], 401);
            }

            $user = JWTAuth::authenticate($token);
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'User not found'], 404);
            }

            $userData = Controller::getUserData($user);
            return response()->json([
                'status' => 1,
                'message' => 'Auto login successful',
                'result' => $userData,
                'token' => $token
            ]);

        } catch (\Tymon\JWTAuth\Exceptions\TokenExpiredException $e) {
            return response()->json(['status' => 0, 'message' => 'Token has expired'], 401);
        } catch (\Tymon\JWTAuth\Exceptions\TokenInvalidException $e) {
            return response()->json(['status' => 0, 'message' => 'Token is invalid'], 401);
        } catch (\Exception $e) {
            return response()->json(['status' => 0, 'message' => 'Token error'], 401);
        }
    }

    public function facebookLogin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'username'          =>'required',
            'email'             =>'required|email',
            'provider_token'    =>'required',
            'provider_id'       =>'required',
            'login_device'      =>'required',
        ]);
        if ($validator->fails())
        {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();
        $email = $input['email'];
        $username = $input['username'];

        $user_token = md5(rand(1, 999999));

        $user = User::where('email',$email)->first();
        if ($user && $user->user_id)
        {
            $updateData = ['user_token' => $user_token];
            $updateData = ['last_login' => Carbon::now()];

            if ($request->has('login_device'))
            {
                $updateData['login_device'] = $request->login_device;
            }

            User::where('user_id',$user->user_id)->update($updateData);

            $userData = Controller::getUserData($user);
            $JWTtoken = JWTAuth::fromUser($user);

            return response()->json(['status' => 1, 'message' => 'You are successfully logged in', 'result' => $userData, 'JWTtoken' => $JWTtoken]);
        }
        else
        {
            $user = User::create([
                'username'=>$username,
                'email'=>$email,
                'provider_name'=>'Facebook',
                'provider_token'=>$request->provider_token,
                'login_device'=>$request->login_device??null,
                'provider_id'=>$request->provider_id,
                'user_token'=>$user_token
            ]);
        }
        if ($user)
        {
            $userData = Controller::getUserData($user);
            $JWTtoken = JWTAuth::fromUser($user);

            return response()->json(['status' => 1, 'message' => 'You are successfully Registred', 'result' => $userData, 'JWTtoken' => $JWTtoken]);
        }
        else
        {
            return response()->json(['status' => 0, 'message' => 'User not registered']);
        }
    }

    public function googleLogin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'username'          =>'required',
            'email'             =>'required|email',
            'provider_token'    =>'required',
            'provider_id'       =>'required',
            'login_device'      =>'required',
        ]);
        if ($validator->fails())
        {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();
        $email = $input['email'];
        $username = $input['username'];

        $user_token = md5(rand(1, 999999));

        $user = User::where('email',$email)->first();

        if ($user && $user->user_id)
        {
            $updateData = ['user_token' => $user_token];
            $updateData = ['last_login' => Carbon::now()];

            if ($request->has('login_device'))
            {
                $updateData['login_device'] = $request->login_device;
            }

            User::where('user_id',$user->user_id)->update($updateData);

            $userData = Controller::getUserData($user);
            $JWTtoken = JWTAuth::fromUser($user);

            return response()->json(['status' => 1, 'message' => 'You are successfully logged in', 'result' => $userData, 'JWTtoken' => $JWTtoken]);
        }
        else
        {
            $user = User::create([
                'username'=>$username,
                'email'=>$email,
                'provider_name'=>'Google',
                'provider_token'=>$request->provider_token,
                'login_device'=>$request->login_device??null,
                'provider_id'=>$request->provider_id,
                'user_token'=>$user_token
            ]);
        }
        if ($user)
        {	
            $userData = Controller::getUserData($user);
            $JWTtoken = JWTAuth::fromUser($user);

            return response()->json(['status' => 1, 'message' => 'You are successfully Registred', 'result' => $userData, 'JWTtoken' => $JWTtoken]);
        }
        else
        {
            return response()->json(['status' => 0, 'message' => 'User not registered']);
        }
    }

    public function appleLogin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'username'          =>'required',
            'email'             =>'required|email',
            'provider_token'    =>'required',
            'provider_id'       =>'required',
            'login_device'      =>'required',
        ]);
        if ($validator->fails())
        {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();
        $email = $input['email'];
        $username = $input['username'];

        $user_token = md5(rand(1, 999999));

        $user = User::where('email',$email)->first();

        if ($user && $user->user_id)
        {
            $updateData = ['user_token' => $user_token];
            $updateData = ['last_login' => Carbon::now()];

            if ($request->has('login_device'))
            {
                $updateData['login_device'] = $request->login_device;
            }

            User::where('user_id',$user->user_id)->update($updateData);

            $userData = Controller::getUserData($user);
            $JWTtoken = JWTAuth::fromUser($user);

            return response()->json(['status' => 1, 'message' => 'You are successfully logged in', 'result' => $userData, 'JWTtoken' => $JWTtoken]);
        }
        else
        {
            $user = User::create([
                'username'=>$username,
                'email'=>$email,
                'provider_name'=>'Apple',
                'provider_token'=>$request->provider_token,
                'login_device'=>$request->login_device??null,
                'provider_id'=>$request->provider_id,
                'user_token'=>$user_token
            ]);
        }
        if ($user)
        {
            $userData = Controller::getUserData($user);
            $JWTtoken = JWTAuth::fromUser($user);

            return response()->json(['status' => 1, 'message' => 'You are successfully Registred', 'result' => $userData, 'JWTtoken' => $JWTtoken]);
        }
        else
        {
            return response()->json(['status' => 0, 'message' => 'User not registered']);
        }
    }

    public function guestLogin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'login_device'   => 'required',
        ]);
        if ($validator->fails())
        {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();
        $login_device = $input['login_device'];

        $user_token = md5(rand(1, 999999));

        $user = User::where('login_device',$login_device)->first();

        if($user)
        {
            $updateData = ['user_token' => $user_token];
            $updateData = ['last_login' => Carbon::now()];

            User::where('user_id',$user->user_id)->update($updateData);

            $userData = Controller::getUserData($user);
            $JWTtoken = JWTAuth::fromUser($user);

            return response()->json(['status' => 1, 'message' => 'You are successfully logged in', 'result' => $userData, 'JWTtoken' => $JWTtoken]);
        }
        else
        {
            $guest_username = 'guest_' . rand(1, 999999);

            $user = User::create([
                'username'=>$guest_username,
                'provider_name'=>'Guest',
                'login_device'=>$login_device,
                'user_token'=>$user_token
            ]);
        }
        if ($user)
        {
            $userData = Controller::getUserData($user);
            $JWTtoken = JWTAuth::fromUser($user);

            return response()->json(['status' => 1, 'message' => 'You are successfully Registred', 'result' => $userData, 'JWTtoken' => $JWTtoken]);
        }
        else
        {
            return response()->json(['status' => 0, 'message' => 'User not registered']);
        }
    }

    public function updateProfile(Request $request)
    {
       $validator = Validator::make($request->all(), [
            'login_device'      => 'required',
            'gold'              => 'nullable|int',
            'solum'             => 'nullable|int',
            'duplicate_shards'  => 'nullable|int',
            'level'             => 'nullable|int',
            'xp'                => 'nullable|int',
            'rating'            => 'nullable|int',
            'collection_score'  => 'nullable|int',
            'win_streak'        => 'nullable|int',
        ]);

        if ($validator->fails()) {
            return response()->json(['status'=>0,'error'=>$validator->errors()], 403);
        }

        $input = $request->all();
        $login_device = $input['login_device'];
        $user = User::where('login_device',$login_device)->first();

        if($user)
        {
            $user_id = $user->user_id;

            if(isset($input['email']))
            {
                $existingUser = User::where('email', $input['email'])->where('user_id', '!=', $user_id)->first();

                if ($existingUser)
                {
                    return response()->json(['status' => 0, 'message' => 'User Registered with this email']);
                }
            }

            $update = [];
            if(isset($input['username']))
            {
                $update['username'] = $input['username'];
            }
            if(isset($input['email']))
            {
                $update['email'] = $input['email'];
            }
            if(isset($input['gold']))
            {
                $update['gold'] = $input['gold'];
            }
            if(isset($input['solum']))
            {
                $update['solum'] = $input['solum'];
            }
            if(isset($input['duplicate_shards']))
            {
                $update['duplicate_shards'] = $input['duplicate_shards'];
            }
            if(isset($input['level']))
            {
                $update['level'] = $input['level'];
            }
            if(isset($input['xp']))
            {
                $update['xp'] = $input['xp'];
            }
            if(isset($input['rating']))
            {
                $update['rating'] = $input['rating'];
            }
            if(isset($input['collection_score']))
            {
                $update['collection_score'] = $input['collection_score'];
            }
            if(isset($input['win_streak']))
            {
                $update['win_streak'] = $input['win_streak'];
            }

            User::where('user_id',$user_id)->update($update);
            
            $userData = Controller::getUserData($user);

            return response()->json(['status' => 1, 'message' => 'Profile Updated Successfully', 'result' => $userData]);
        }
        else
        {
            return response()->json(['status' => 0, 'message' => 'You are not authorized please login again'],401);
        }
    }

    public function userPurchase(Request $request)
    {
       $validator = Validator::make($request->all(), [
            'user_id'           => 'required|exists:users,user_id',
            'product_id'        => 'required|exists:products,product_id',
            'purchase_token'    => 'required',
            'platform'          => 'required',
            'verified'          => 'required',
            'reward_granted'    => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json(['status'=>0,'error'=>$validator->errors()], 403);
        }

        $input = $request->all();
        $user_id = $input['user_id'];
        $product_id = $input['product_id'];
        $user = User::where('user_id',$user_id)->first();

        if($user)
        {
            $pro = User_purchase::where('user_id',$user_id)->where('product_id',$product_id)->first();
            if($pro)
            {
                return response()->json(['status' => 0, 'message' => 'Already this product purchase'],401);
            }
            else
            {
                $purchase = User_purchase::create($input);
                return response()->json(['status' => 1, 'message' => 'User Purchase Successfully', 'result' => $purchase]);   
            }
        }
        else
        {
            return response()->json(['status' => 0, 'message' => 'You are not authorized please login again'],401);
        }
    }

    public function userHistory(Request $request)
    {
       $validator = Validator::make($request->all(), [
            'user_id'           => 'required|exists:users,user_id',
        ]);

        if ($validator->fails()) {
            return response()->json(['status'=>0,'error'=>$validator->errors()], 403);
        }

        $input = $request->all();
        $user_id = $input['user_id'];
        $user = User::where('user_id',$user_id)->first();

        if($user)
        {
            $pro = User_purchase::where('user_id',$user_id)->get();
            if($pro->count())
            {
                return response()->json(['status' => 1, 'message' => 'User History', 'result' => $pro]);
            }
            else
            {
                return response()->json(['status' => 0, 'message' => 'No History'],401);
            }
        }
        else
        {
            return response()->json(['status' => 0, 'message' => 'You are not authorized please login again'],401);
        }
    }

	private function getRules($type, $input) {
        $return = array();
        if($type=='login')
        {
            $return['email'] = 'required|email|max:100';
            $return['password'] = 'required|max:20';
        }
        else
        if($type=='register')
        {
            $return['email'] = 'required|email|max:100|unique:users,email';
            $return['password'] = 'required|max:20';
        }
        return $return;
    }

    private function messages() {
        return [
            'email.required'  => $this->getRequiredMessage('email'),
            'email.max'  => $this->getGreaterMessage('email ', 100),
        ];
    }

    private function getRequiredMessage($string) {
        return 'The ' . $string . ' field is required.';
    }

    private function getGreaterMessage($string, $maxchar) {
        return 'The ' . $string . ' may not be greater than ' . $maxchar . ' characters.';
    }
}
