<?php
namespace App\Http\Controllers\api;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
//use Illuminate\Support\Facades\Mail;
use Mail;
use App\Models\User;
use App\Models\Country;
use App\Models\Staticpage;
use Exception;
use Validator;
use Log;
use Carbon\Carbon;

class UserController extends Controller
{
    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'username'          => 'required|min:3',
            'email'             => 'required',
            'countryId'        => 'required|exists:countries,id',
            'contactNumber'     => 'required|min:10',
            'password'          => 'required|min:6',
            'profilePicture'    => 'required',
        ]);

        if ($validator->fails())
        {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();

        $input['password'] = bcrypt($input['password']);
        $input['user_token'] = md5(rand(1, 999999));

        $email = $input['email'];
        $contactNumber = $input['contactNumber'];

        $countryId = $input['countryId'];
        $country = Country::where('id', $countryId)->first();
        $contact_number_new = $country->phone_code.$contactNumber;

        $filename = '';
        $upload_dir_path = public_path()."/uploads/userImage";
        if(isset($input['profilePicture']))
        {
            $file = $input['profilePicture'];
            $filename = rand(0000,9999).$file->getClientOriginalName();
            $file->move($upload_dir_path, $filename);
        }

        $input['profilePicture'] = $filename;

        if(isset($input['email']))
        {
            if ($this->isUserExists($email))
            {
                return response()->json(['status' => 0, 'message' => 'User already registered with this email']);
            }
        }

        if ($this->isUserPhoneExists($contactNumber, $countryId))
        {
            return response()->json(['status' => 0, 'message' => 'User already registered with this phone number & country code']);
        }

        if ($this->isUserUsernameExists($input['username']))
        {
            return response()->json(['status' => 0, 'message' => 'User already registered with this username']);
        }

        $user = User::create($input);

        if ($user->id > 0)
        {
            $userData = Controller::getUserData($user);

            $mail = $input['email'];
            $data['username']=$user->username;

            Mail::send('adminpanel.welcomeEmail', $data,function($message) use($mail)
            {
                $message->from('kitest@keshavinfotechdemo2.com', 'Abhi-Game');
                $message->to($mail)->subject('Welcome to Abhi-Game!');
            });

            return response()->json(['status' => 1, 'message' => 'User Registered Successfully', 'result' => $userData]);
        }
        else
        {
            return response()->json(['status' => 0, 'message' => 'User not registered']);
        }
    }

    private function isUserExists($email)
    {
        return User::where('email', $email)->exists();
    }

    private function isUserPhoneExists($contactNumber,$countryId)
    {
        return User::where('contactNumber', $contactNumber)->where('countryId', $countryId)->exists();
    }

    private function isUserUsernameExists($username)
    {
        return User::where('username', $username)->exists();
    }

    public function login(Request $request)
    {
        $login = $request->input('login');
        $password = $request->input('password');
        $countryId = $request->input('countryId');

        // Determine if login is by phone or username
        $fieldType = filter_var($login, FILTER_VALIDATE_EMAIL) ? 'email' : (is_numeric($login) ? 'contactNumber' : 'username');

        if($fieldType == 'email')
        {
            $validator = Validator::make($request->all(), [
                'login'         => 'required',
                'password'      => 'required'
            ]);
        }
        else if($fieldType == 'contactNumber')
        {
            $validator = Validator::make($request->all(), [
                'login'         => 'required',
                'countryId'    => 'required|exists:countries,id',
                'password'      => 'required'
            ]);
        }
        else
        {
            $validator = Validator::make($request->all(), [
                'login'         => 'required',
                'password'      => 'required'
            ]);
        }

        if ($validator->fails())
        {
            return response()->json(['error'=>$validator->errors()], 403);
        }

        if($countryId)
        {
            if (Auth::guard('web')->attempt([$fieldType => $login, 'countryId' => $countryId, 'password' => $password], false))
            {
                $user = Auth::user();
                $userData = Controller::getUserData($user);
                
                    $tkn = md5(rand(1, 999999));
                    $user->update(['user_token' => $tkn,]);
                    $userData['user_token'] = $tkn;

                    return response()->json(['status' => 1, 'message' =>"Login Successfully",'result'=>$userData]);
            }
        }
        else
        {
            if (Auth::guard('web')->attempt([$fieldType => $login, 'password' => $password], false))
            {
                $user = Auth::user();
                $userData = Controller::getUserData($user);
                
                    $tkn = md5(rand(1, 999999));
                    $user->update(['user_token' => $tkn,]);
                    $userData['user_token'] = $tkn;

                    return response()->json(['status' => 1, 'message' =>"Login Successfully",'result'=>$userData]);
            }
        }
        return response()->json(['status' => 0, 'message' => "Username or Password is incorrect. Please try again!"]);
    }

    public function updateProfile(Request $request)
    {
       $validator = Validator::make($request->all(), [
            'user_id'        => 'required',
            'user_token'     => 'required',
            'username'      => 'required',
            'email'         => 'required',
            'contactNumber' => 'required|min:10',
            'profilePicture'    => 'required',
        ]);

        if ($validator->fails()) {
            return response()->json(['status'=>0,'error'=>$validator->errors()], 403);
        }
        //-------------------Message------------------------------
            $msg1 = "Profile Updated Successfully";
            $msg2 = "Profile not Updated";
            $msg3 = "You are not authorized please login again";
            $msg4 = "User Registered with this email or Phone Number";
        //-------------------Message------------------------------

        $input = $request->all();
        $user_id = $input['user_id'];
        $user_token = $input['user_token'];

        $verifieduser_id = Controller::checkToken($user_id, $user_token);

        if ($verifieduser_id == $user_id) {
            $user = User::find($user_id);

            if (!$user) {
                return response()->json(['status' => 0, 'message' => $msg3],401);
            }

            // Check if the provided email or phone already exists for other users
            $existingUser = User::where('email', $input['email'])->where('id', '!=', $user_id)->first();

            if ($existingUser) {
                return response()->json(['status' => 0, 'message' => $msg4]);
            }

            $upload_dir_path = public_path()."/uploads/userImage";
            if(isset($input['profilePicture']))
            {
                $file = $input['profilePicture'];
                $filename = rand(0000,9999).$file->getClientOriginalName();
                $file->move($upload_dir_path, $filename);
            }

            $update = [
                'username' => $input['username']
            ];

            if (isset($input['email'])) {
                $update['email'] = $input['email'];
            }

            if (isset($input['contactNumber'])) {
                $update['contactNumber'] = $input['contactNumber'];
            }

            if(isset($input['profilePicture']))
            {
                $update['profilePicture'] = $filename;
            }

            // Update the user's profile
            if ($user->update($update))
            {
                $userData = Controller::getUserData($user);

                return response()->json(['status' => 1, 'message' => $msg1, 'result' => $userData]);
            } else {
                return response()->json(['status' => 0, 'message' => $msg2]);
            }
        }
        else
        {
            return response()->json(['status' => 0, 'message' => $msg3],401);
        }
    }

    public function forgotPassword(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['error'=>$validator->errors()], 403);
        }

        $email = $request->input('email');

        $user = User::where('email', $email)->first();

        if ($user)
        {
            $newPassword = rand(100000, 999999);
            $user->update(['password' => bcrypt($newPassword)]);

            $mail = $email;
            $data['username']=$user->username;
            $data['newPassword']=$newPassword;

            Mail::send('adminpanel.test', $data,function($message) use($mail){
                $message->from('kitest@keshavinfotechdemo2.com', 'Abhi-Game');
                $message->to($mail)->subject('Forgot Password');
            });

            $success = [
                'user_id' => $user->id,
                'email' => $user->email,
            ];

            return response()->json(['status' => 1, 'message' => "Mail sent Successfully", 'result' => $success]);
        } else {
            return response()->json(['status' => 0, 'message' => "Mail not Found"]);
        }
    }

    public function changePassword(Request $request)
    {
        $requestData = $request->all();

        $validator = Validator::make($requestData, [
            'user_id' => 'required',
            'user_token' => 'required',
            'oldPassword'=> 'required|min:6',
            'newPassword'=> 'required|min:6',
        ]);

        if ($validator->fails())
        {
            return response()->json(['status'=>0,'error'=>$validator->errors()], 403);
        }

        /*-------------------Message------------------------------*/
        $msg1 = "Password changed successfully";
        $msg2 = "Password not changed";
        $msg3 = "Old Password does not match";
        $msg4 = "You are not authorized please login again";
        /*-------------------Message------------------------------*/

        $user_id = (int)$requestData['user_id'];
        $user_token = $requestData['user_token'];
        $oldPassword = $requestData['oldPassword'];
        $userData['password'] = bcrypt($requestData['newPassword']);

        $verifieduser_id = Controller::checkToken($user_id, $user_token);

        if ($verifieduser_id == $user_id)
        {
            $user = User::where('id',$user_id)->get();
            if (Hash::check($oldPassword, $user[0]->password))
            {
                $changePassword = User::where('id',$user_id)->update($userData);

                if($changePassword){
                    return response()->json(['status'=>1,'message'=>$msg1]);
                }
                else {
                    return response()->json(['status'=>0,'message'=>$msg2], 403);
                }
            }
            else{
                return response()->json(['status'=>0,'message'=>$msg3], 403);
            }
        }
        else
        {
            return response()->json(['status' => 0, 'message' => $msg4], 401);
        }
    }

    public function deleteAccount(Request $request)
    {
        $requestData = $request->all();
        $validator = Validator::make($requestData, [
            'user_id' => 'required',
            'user_token' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json(['status'=>0,'error'=>$validator->errors()], 403);
        }

        /*-------------------Message------------------------------*/
        $msg1 = "Your account deleted successfully";
        $msg2 = "Your account not deleted";
        $msg3 = "You are not authorized please login again";
        /*-------------------Message------------------------------*/

        $user_id = (int)$requestData['user_id'];
        $user_token = $requestData['user_token'];

        $verifieduser_id = Controller::checkToken($user_id, $user_token);

        if ($verifieduser_id == $user_id)
        {
            $deleteAccount = User::where('id',$user_id)->update($update);

            if($deleteAccount)
            {
                return response(['status'=>1,'message'=>$msg1], 200);
            }
            else {
                return response(['status'=>0,'message'=>$msg2], 403);
            }
        }
        else {
            return response(['status'=>0,'message'=>$msg3], 401);
        }
    }

    public function logout(Request $req)
	{
		$validator = Validator::make($req->all(), [
            'user_id'    => 'required',
            'user_token' =>'required',
        ]);
		if ($validator->fails()) {
            return response()->json(['status'=>1,'message'=>$validator->errors()]);
        }
		$data = User::where('id', $req->user_id)->where('user_token',$req->user_token)->get();
        if($data->count())
		{
			$update['user_token']= '';
			User::where('id', $req->user_id)->update($update);
            return response()->json(['status'=>1,'message'=>'Logout successfully']);
            exit;
        }
        else
        {
            return response()->json(['status'=>0,'message'=>'Logout failed']);
            exit;
        }
	}
}
